
/**
 *  Download and install the app-ng-runtime package
 *  Run the build and post build for the generated angular app.
 *  
 *  CONSOLE ARGUMENTS:-
 *  nodeVMArgs:String: Node environment params  (Eg: '--max-old-space-size=2040')
 *  ngBuildParams:String: Anglar build params (Eg: '--prod=true --source-map=false')
 *  runtimeUIVersion:String: Runtime version (Eg: 10.6.6-next.10243)
 *  appSrc:String: Source folder to generate the angular app (current directory Eg: '.')
 *  appTarget:String: Target folder to save the generated app (Eg: 'target/ui-build/generated-app')
 *  codegenPath:String: Input to the codegen-CLI -->  Codegen installation path to copy the angular-template-app
 *  optimizeUIBuild : By default this flag will be true. If it's windows platform we are making the flag to false, it means no optimisation for windows.
 */
const fs = require('fs');
const { executeSyncCmd,  MSG_NG_RUNTIME_LOG, MSG_NG_RUNTIME_SUCCESS } = require('./build-util');
const os = require("os");

const NPM_PACKAGE_SCOPE = '@wavemaker';

/**
 * @TODO: This is a temporary workaround to extract deploy-url from 'ngBuildParams', need to be replaced in future.
 */
const updateDeployUrl = (args) => {
    let buildArgs = args.ngBuildParams.split(' ');
    const deployParam = buildArgs.find(i => i.startsWith("--deploy-url="));
    if(!deployParam) {
        buildArgs.push("--deploy-url=ng-bundle/");
    } else {
        buildArgs.filter((param, index) => {
            if (param.includes('--deploy-url=_cdnUrl_')) {
                buildArgs[index] = "--deploy-url=_cdnUrl_/ng-bundle/"
                return true;
            }
        });
    }
    buildArgs.push(args.nodeVMArgs);
    return buildArgs.join(" ");
};

/**
 * Download the app runtime package
 * Symlink to reuse the existing node modules (If windows optimizeUIBuild will be false and install all node modules every build )
 * Run the ng build and post build for the project
 */
const buildAngularApp = (args) => {
    const HOME_DIR = os.homedir(), ANGULAR_APP_TEMPLATE_NAME = "angular-app";
    const TARBALL_DOWNLOAD_PATH = (HOME_DIR + '/.wm/node_modules') + '/' + ANGULAR_APP_TEMPLATE_NAME + '/' + args.runtimeUIVersion;
    const PACKAGE_NAME = NPM_PACKAGE_SCOPE + '/' + ANGULAR_APP_TEMPLATE_NAME + '@' + args.runtimeUIVersion;
    const TARBALL_NAME = 'wavemaker' + '-' + ANGULAR_APP_TEMPLATE_NAME + '-' + args.runtimeUIVersion + '.tgz';

    fs.mkdirSync(TARBALL_DOWNLOAD_PATH, { recursive: true });
    //npm pack will download the given package as a tar file
    executeSyncCmd('cd ' + TARBALL_DOWNLOAD_PATH + ` && npm pack ${PACKAGE_NAME}`, null, `Downloading tarball - ${PACKAGE_NAME} to the location - ${TARBALL_DOWNLOAD_PATH}`);
    executeSyncCmd('cd ' + TARBALL_DOWNLOAD_PATH + ` && tar -xvf ${TARBALL_NAME}`, null, `Unzipping the tarball - ${TARBALL_NAME}`);

    let packageInfo = {
        scope: NPM_PACKAGE_SCOPE,
        name: ANGULAR_APP_TEMPLATE_NAME,
        version: args.runtimeUIVersion,
        packageJsonFile: `${TARBALL_DOWNLOAD_PATH}/package/package.json`,
        packageJsonLockFile: `${TARBALL_DOWNLOAD_PATH}/package/package-lock.json`,
        successMsg: MSG_NG_RUNTIME_SUCCESS,
        infoMsg: MSG_NG_RUNTIME_LOG
    }

    try {
        if (args.optimizeUIBuild) {
            fs.copyFileSync(TARBALL_DOWNLOAD_PATH + `/package/package.json`, TARBALL_DOWNLOAD_PATH + '/package.json');
            fs.copyFileSync(TARBALL_DOWNLOAD_PATH + `/package/package-lock.json`, TARBALL_DOWNLOAD_PATH + '/package-lock.json');
            fs.copyFileSync(TARBALL_DOWNLOAD_PATH + `/package/.npmrc`, TARBALL_DOWNLOAD_PATH + '/.npmrc');

            // Download angular-app node_modules
            executeSyncCmd('cd ' + TARBALL_DOWNLOAD_PATH + ' && npm install ', null, "Installing app dependencies...");

            // Symlink to reuse the existing node_modules
            fs.symlinkSync(TARBALL_DOWNLOAD_PATH + `/node_modules`, args.appTarget + '/node_modules', 'dir');
        } else {
            fs.copyFileSync(TARBALL_DOWNLOAD_PATH + `/package/package/package.json`, args.appTarget + '/package.json');
            fs.copyFileSync(TARBALL_DOWNLOAD_PATH + `/package/package-lock.json`, args.appTarget + '/package-lock.json');
            fs.copyFileSync(TARBALL_DOWNLOAD_PATH + `/package/.npmrc`, args.appTarget + '/.npmrc');
            executeSyncCmd('cd ' + args.appTarget + ` && npm install `, null, MSG_NG_RUNTIME_LOG);
        }
    } catch (err) {
        console.log(MSG_NG_RUNTIME_LOG + 'FAILED the symlink for the node_modules', err);
        process.exit(err.code || err.pid);
    }

    let ngBuildParams = updateDeployUrl(args);
    const NG_BUILD_MSG = 'NG BUILD MIGHT HAVE FAILED WITH HEAP OUT OF MEMORY, RE-EXECUTE THE BUILD BY INCREASING THE MAX-OLD-SPACE-SIZE ARGUMENT VALUE FOR BUILD.UI.NODE.ARGS PROPERTY IN DEPLOYMENT PROFILE';
    // Generating the angular build and post build process.
    executeSyncCmd('cd ' + args.appTarget + ' && node ./build-scripts/build.js ' + ngBuildParams, null, MSG_NG_RUNTIME_LOG, false, NG_BUILD_MSG);
}


module.exports = {
    buildAngularApp
}
